﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.CodeDom.Compiler;
using System.Reflection;

namespace howto_codedom_evaluate_numeric_expressions
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // Evaluate the expression.
        private void btnEvaluate_Click(object sender, EventArgs e)
        {
            // Turn the equation into a function.
            string function_text =
                "public static class Evaluator" +
                "{" +
                "    public static double Evaluate(double x, double y)" +
                "    {" +
                "        return " + txtExpression.Text + ";" +
                "    }" +
                "}";

            // Compile the function.
            CodeDomProvider code_provider = CodeDomProvider.CreateProvider("C#");

            // Generate a non-executable assembly in memory.
            CompilerParameters parameters = new CompilerParameters();
            parameters.GenerateInMemory = true;
            parameters.GenerateExecutable = false;

            // Compile the code.
            CompilerResults results =
                code_provider.CompileAssemblyFromSource(parameters, function_text);

            // If there are errors, display them.
            if (results.Errors.Count > 0)
            {
                string msg = "Error compiling the expression.";
                foreach (CompilerError compiler_error in results.Errors)
                {
                    msg += "\n" + compiler_error.ErrorText;
                }
                MessageBox.Show(msg, "Expression Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                // Get the Evaluator class type.
                Type evaluator_type = results.CompiledAssembly.GetType("Evaluator");

                // Get a MethodInfo object describing the Evaluate method.
                MethodInfo method_info = evaluator_type.GetMethod("Evaluate");

                // Make the parameter list.
                object[] method_params = new object[]
                {
                    double.Parse(txtX.Text),
                    double.Parse(txtY.Text)
                };

                // Execute the method.
                double expression_result =
                    (double)method_info.Invoke(null, method_params);

                // Display the returned result.
                txtResult.Text = expression_result.ToString();
            }
        }
    }
}
